
# q-expansion of newform 4050.2.a.g, downloaded from the LMFDB on 09 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 4050
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1], [0], [0], [-1], [2], [-6], [2], [6], [-1], [-9], [-2], [2], [11], [-4], [7], [0], [4], [-7], [-11], [6], [-4], [-12], [11], [-1], [-8], [-2], [-8], [-3], [7], [-12], [-19], [-12], [-12], [16], [1], [10], [4], [4], [3], [4], [-2], [-13], [-6], [-10], [-8], [-18], [18], [23], [8], [-7], [10], [-28], [1], [-18], [-12], [-16], [-3], [-14], [-22], [3], [1], [18], [-9], [-6], [-22], [2], [-8], [8], [12], [-11], [16], [-30], [-16], [12], [-16], [-32], [-19], [-4], [10], [38], [-34], [22], [-16], [2], [24], [9], [-18], [-10], [21], [36], [-36], [-28], [-12], [0], [-24], [-27], [15], [37], [-29], [-17], [35], [24], [37], [2], [-20], [32], [-3], [30], [12], [-22], [1], [-34], [32], [-10], [8], [-13], [-33], [-33], [26], [-20], [-10], [6], [-22], [-4], [-44], [-13], [-27], [44], [21], [4], [40], [15], [26], [-10], [9], [-15], [-12], [44], [2], [30], [-20], [31], [15], [35], [15], [-38], [46], [18], [14], [45], [46], [15], [3], [0], [33], [-12], [-36], [18], [52], [41], [51], [-2], [-17], [42], [-54], [19], [4], [0], [17], [-18], [48], [-9], [-34], [-44], [-32], [-42], [-28], [30], [-7], [35], [32], [58], [22], [-6], [49], [39], [-16], [-11], [23], [-8], [66], [5], [-50], [15], [15], [-66], [-15], [48], [-6], [-65], [30], [6], [-24], [1], [-40], [-42], [-6], [-25], [-5], [-44], [-32], [66], [55], [-12], [-38], [49], [43], [46], [31], [68], [57], [58], [63], [25], [27], [6], [62], [10], [-35], [-10], [40], [22], [2], [-45], [-61], [41], [-69], [-50], [-18], [-30], [3], [50], [-1], [49], [-58], [-6], [-69], [-42], [38], [59], [-42], [-55], [-68], [-41], [-56], [60], [10], [-23], [-62], [-38], [-21], [-49], [-50], [-68], [-18], [20], [-6], [17], [47], [24], [-46], [19], [-14], [-8], [-64], [31], [19], [-10], [-31], [-78], [0], [-10], [4], [-5], [55], [-52], [78], [4], [-40], [5], [-82], [-12], [54], [-30], [26], [45], [-66], [72], [56], [-12], [-50], [48], [2], [-48], [-16], [-70], [-47], [53], [-62], [-36], [0], [3], [66], [-39], [56], [29], [-8], [-50], [-64], [78], [20], [22], [42], [-28], [-72], [17], [30], [21], [-85], [-88], [37], [74], [-4], [-92], [-36], [-45], [31], [36], [1], [-16], [-24], [52], [-19], [-76], [-34], [0], [54], [-53], [28], [-24], [-28], [54], [63], [9], [65], [78], [-48], [84], [44], [-67], [44], [-15], [36], [86], [-84], [-10], [66], [-9], [-78], [46], [8], [-49], [-27], [-60], [12], [50], [-44], [12], [-3], [82], [81], [-86], [-35], [-50], [-30], [-24], [18], [-44], [33], [77], [-37], [42], [48], [-68], [64], [-40], [-16], [46], [-40], [-42], [-53], [20], [88], [-42], [21], [62], [-36], [21], [49], [24], [-8], [-36], [-27], [52], [38], [-67], [-58], [48], [-28], [-15], [93], [-20], [-16], [12], [-86], [84], [73], [58], [-18], [32]]
